
        /*
         * Vectorテーブルとジャンプテーブル
         * サイズは合計0x3c
         */
        .section .vector,"a"
        .code 32
        .align 4
        .global vector_table
vector_table:
        ldr pc, reset_vector       /* リセット             */
        ldr pc, undef_vector       /* 未定義命令           */
        ldr pc, swi_vector         /* ソフトウェア割込み   */
        ldr pc, prefech_vector     /* プリフェッチアボード */
        ldr pc, data_abort_vector  /* データアボード       */
        ldr pc, reset_vector
        ldr pc, irq_vector         /* IRQ                  */
        ldr pc, fiq_vector         /* IRQ                  */

/*
 *  ベクターテーブル
 */
reset_vector:
        .long   start
undef_vector:
        .long   undef_exception
swi_vector:
        .long   swi_exception
prefech_vector:
        .long   prefetch_exception
data_abort_vector:
        .long   data_abort_exception
irq_vector:
        .long   irq_exception
fiq_vector:
        .long   fiq_exception


#define VICAddress 0xFFFFFF00
		.text
        .code 32
        .align 4
irq_exception:
        /*
         * 割込みモード
         *
         * cpsrがspsr_irqに復帰先がr14_irq(lp)に入り，
         * spsr_irqとr14_irqとr13(sp)_irqが r14,r13となる．
         */

        stmdb   sp!, {r0-r12,r14} 	/* コンテキスト退避 */
        mrs		r0, spsr			/* PSR退避 */
        stmdb	sp!, {r0}
         /*
         *  ハンドラの呼び出し
         */
		ldr		r4,=VICAddress
		ldr		r5,[r4]					/* VICAddressレジスタから、割り込みハンドラのアドレス取得*/
		ldr		r14,=irq_return			/* bl 命令を使えないので、自分で戻り番地を設定する */
		mov		pc,r5;					/* VICAddressに提示されたアドレスをcall */
         /*
         *  復帰処理
         */
irq_return:
		str		r5,[r4]					/* VICAddressへ書き込んで割り込みクリア */
        ldmia   sp!,{r0} 				/* spsr を取り出す */
		msr		spsr_cf, r0
		ldmia	sp!,{r0-r12,r14}		/* コンテキスト復帰 */
		subs	pc, r14,#4				/* IRQからの復帰 */

reset_exception:
undef_exception:
swi_exception:
prefetch_exception:
data_abort_exception:
reserved_exception:
fiq_exception:
		b fiq_exception


